sub require_mail
{
return if ($require_mail++);
if ($config{'mail_system'} == 1) {
	# Using sendmail for email
	&foreign_require("sendmail", "sendmail-lib.pl");
	&foreign_require("sendmail", "virtusers-lib.pl");
	&foreign_require("sendmail", "aliases-lib.pl");
	&foreign_require("sendmail", "boxes-lib.pl");
	%sconfig = &foreign_config("sendmail");
	$sendmail_conf = &sendmail::get_sendmailcf();
	$sendmail_vfile = &sendmail::virtusers_file($sendmail_conf);
	($sendmail_vdbm, $sendmail_vdbmtype) =
		&sendmail::virtusers_dbm($sendmail_conf);
	$sendmail_afiles = &sendmail::aliases_file($sendmail_conf);
	}
elsif ($config{'mail_system'} == 0) {
	# Using postfix for email
	&foreign_require("postfix", "postfix-lib.pl");
	&foreign_require("postfix", "boxes-lib.pl");
	%pconfig = &foreign_config("postfix");
	$virtual_type = $postfix::virtual_maps || "virtual_maps";
	local $gvfunc = defined(&postfix::get_real_value) ?
				\&postfix::get_real_value :
				\&postfix::get_current_value;
	$virtual_maps = &$gvfunc($virtual_type);
	$postfix_afiles = [ &postfix::get_aliases_files(
				&$gvfunc("alias_maps")) ];
	}
elsif ($config{'mail_system'} == 2) {
	# Using qmail for email
	&foreign_require("qmailadmin", "qmail-lib.pl");
	%qmconfig = &foreign_config("qmailadmin");
	$can_alias_types{2} = 0;
	$can_alias_types{7} = 0;
	}
}

# list_domain_aliases(&domain)
sub list_domain_aliases
{
&require_mail();
local $g = $_[0]->{'group'};
local ($u, %foruser);
foreach $u (&list_domain_users($_[0])) {
	local $pop3 = &remove_userdom($u->{'user'}, $_[0]);
	$foruser{$pop3."\@".$_[0]->{'dom'}} = $u->{'user'};
	}
if ($d->{'mailbox'}) {
	$foruser{$d->{'user'}."\@".$_[0]->{'dom'}} = $d->{'user'};
	}
local @virts = &list_virtusers();
return grep { $_->{'from'} =~ /\@(\S+)$/ && $1 eq $_[0]->{'dom'} &&
	      ($foruser{$_->{'from'}} ne $_->{'to'}->[0] ||
	       @{$_->{'to'}} != 1) } @virts;
}

# setup_mail(&domain)
# Adds a domain to the list of those accepted by the mail system
sub setup_mail
{
&$first_print($text{'setup_doms'});
&require_mail();
if ($config{'mail_system'} == 1) {
	# Just add to sendmail local domains file
	local $cwfile = &sendmail_locals_file();
	open(LOCALS, ">>$cwfile");
	print LOCALS $_[0]->{'dom'},"\n";
	close(LOCALS);
	&sendmail::restart_sendmail();
	}
elsif ($config{'mail_system'} == 0) {
	# Add a special postfix virtual entry just for the domain
	&create_virtuser({ 'from' => $_[0]->{'dom'}, 'to' => [ 'anything' ] });
	}
elsif ($config{'mail_system'} == 2) {
	# Add to qmail locals file, rcpthosts file  and virtualdomains file
	local $dlist = &qmailadmin::list_control_file("locals");
	push(@$dlist, $_[0]->{'dom'});
	&qmailadmin::save_control_file("locals", $dlist);

	local $rlist = &qmailadmin::list_control_file("rcpthosts");
	push(@$rlist, $_[0]->{'dom'});
	&qmailadmin::save_control_file("rcpthosts", $rlist);

	local $virtmap = { 'domain' => $_[0]->{'dom'},
			   'prepend' => $_[0]->{'group'} };
	&qmailadmin::create_virt($virtmap);
	&qmailadmin::restart_qmail();
	}
&$second_print($text{'setup_done'});
}

# delete_mail(&domain)
# Removes a domain from the list of those accepted by the mail system
sub delete_mail
{
&$first_print($text{'delete_doms'});
&require_mail();
if ($config{'mail_system'} == 1) {
	# Delete domain from sendmail local domains file
	local $cwfile = &sendmail_locals_file();
	local @locals;
	open(LOCALS, $cwfile);
	while(<LOCALS>) {
		s/\r|\n//;
		push(@locals, "$_\n") if ($_ ne $_[0]->{'dom'});
		}
	close(LOCALS);
	open(LOCALS, ">$cwfile");
	print LOCALS @locals;
	close(LOCALS);
	&sendmail::restart_sendmail();
	}
elsif ($config{'mail_system'} == 0) {
	# Delete the special postfix virtuser
	local @virts = &list_virtusers();
	local ($lv) = grep { $_->{'from'} eq $_[0]->{'dom'} } @virts;
	if ($lv) {
		&delete_virtuser($lv);
		}
	local @md = split(/[, ]+/,&postfix::get_current_value("mydestination"));
	local $idx = &indexof($_[0]->{'dom'}, @md);
	if ($idx >= 0) {
		# Delete old-style entry too
		splice(@md, $idx, 1);
		&postfix::set_current_value("mydestination", join(", ", @md));
		&shutdown_mail_server();
		&startup_mail_server();
		}
	}
elsif ($config{'mail_system'} == 2) {
	# Delete domain from qmail locals file, rcpthosts file and virtuals
	local $dlist = &qmailadmin::list_control_file("locals");
	$dlist = [ grep { $_ ne $_[0]->{'dom'} } @$dlist ];
	&qmailadmin::save_control_file("locals", $dlist);

	local $rlist = &qmailadmin::list_control_file("rcpthosts");
	$rlist = [ grep { $_ ne $_[0]->{'dom'} } @$rlist ];
	&qmailadmin::save_control_file("rcpthosts", $rlist);

	local ($virtmap) = grep { $_->{'domain'} eq $_[0]->{'dom'} &&
				  !$_->{'user'} } &qmailadmin::list_virts();
	&qmailadmin::delete_virt($virtmap) if ($virtmap);
	&qmailadmin::restart_qmail();
	}
&$second_print($text{'setup_done'});
}

# modify_mail(&domain, &olddomain)
# Deal with a change in domain name
sub modify_mail
{
if ($_[0]->{'dom'} ne $_[1]->{'dom'}) {
	&delete_mail($_[1]);
	&setup_mail($_[0]);
	}
}

# disable_mail(&domain)
# Same as deleting ..
sub disable_mail
{
&delete_mail($_[0]);
}

# enable_mail(&domain)
# Same as setting up ..
sub enable_mail
{
&setup_mail($_[0]);
}

# check_mail_clash()
# Does nothing, because no clash checking is needed
sub check_mail_clash
{
return 0;
}

# is_local_domain(domain)
# Returns 1 if some domain is used for mail on this system, 0 if not
sub is_local_domain
{
local $found = 0;
if ($config{'mail_system'} == 1) {
	# Check Sendmail local domains file
	local $cwfile = &sendmail_locals_file();
	open(CW, $cwfile);
	while(<CW>) {
		s/\r|\n//;
		$found++ if ($_ eq $_[0]);
		}
	close(CW);
	}
elsif ($config{'mail_system'} == 0) {
	# Check Postfix virtusers and mydestination
	local @virts = &list_virtusers();
	local ($lv) = grep { $_->{'from'} eq $_[0] } @virts;
	$found++ if ($lv);
	local @md = split(/[, ]+/,&postfix::get_current_value("mydestination"));
	$found++ if (&indexof($_[0], @md) >= 0);
	}
elsif ($config{'mail_system'} == 2) {
	# Check qmail local domains file
	local $dlist = &qmailadmin::list_control_file("locals");
	$found++ if (&indexof($_[0], @$dlist) >= 0);
	}
return $found;
}

sub sendmail_locals_file
{
local $conf = &sendmail::get_sendmailcf();
local ($f, $cwfile);
foreach $f (&sendmail::find_type("F", $conf)) {
	if ($f->{'value'} =~ /^w[^\/]*(\/\S+)/ ||
	    $f->{'value'} =~ /^\{w\}[^\/]*(\/\S+)/) { $cwfile = $1; }
	}
return $cwfile;
}

# list_virtusers()
# Returns a list of a virtual mail address mappings. Each may actually have
# an alias as its destination, and is automatically expanded to the destinations
# for that alias.
sub list_virtusers
{
# Build list of unix users, to exclude aliases with same name as users
# (which are picked up by list_domain_users instead).
&require_mail();
if (!defined(%unix_users)) {
	local $u;
	setpwent();
	while($u = getpwent()) {
		$unix_user{$u}++;
		}
	endpwent();
	}

if ($config{'mail_system'} == 1) {
	# Get from sendmail
	local @svirts = &sendmail::list_virtusers($sendmail_vfile);
	local %aliases = map { $_->{'name'}, $_ }
			 grep { $_->{'enabled'} && !$unix_user{$_->{'name'}} }
				&sendmail::list_aliases($sendmail_afiles);
	local ($v, $a, @virts);
	foreach $v (@svirts) {
		local %rv = ( 'virt' => $v,
			      'from' => $v->{'from'} );
		if ($v->{'to'} !~ /\@/ && ($a = $aliases{$v->{'to'}})) {
			# Points to an alias - use its values
			$rv{'to'} = $a->{'values'};
			$rv{'alias'} = $a;
			}
		else { 
			# Just the original value
			$rv{'to'} = [ $v->{'to'} ];
			}
		push(@virts, \%rv);
		}
	return @virts;
	}
elsif ($config{'mail_system'} == 0) {
	# Get from postfix
	local $svirts = &postfix::get_maps($virtual_type);
	local %aliases = map { $_->{'name'}, $_ }
			 grep { $_->{'enabled'} && !$unix_user{$_->{'name'}} }
			     &postfix::list_aliases($postfix_afiles);
	local ($v, $a, @virts);
	foreach $v (@$svirts) {
		local %rv = ( 'from' => $v->{'name'},
			      'virt' => $v );
		if ($v->{'value'} !~ /\@/ && ($a = $aliases{$v->{'value'}})) {
			$rv{'to'} = $a->{'values'};
			$rv{'alias'} = $a;
			}
		else {
			$rv{'to'} = [ $v->{'value'} ];
			}
		push(@virts, \%rv);
		}
	return @virts;
	}
elsif ($config{'mail_system'} == 2) {
	# Find all qmail aliases like .qmail-group-user
	local @virtmaps = grep { !$_->{'user'} } &qmailadmin::list_virts();
	local @aliases = &qmailadmin::list_aliases();
	local ($an, $v, @virts);
	foreach $an (@aliases) {
		# Find domain in virtual maps
		local $a = &qmailadmin::get_alias($an);
		local $name = $a->{'name'};
		foreach $v (@virtmaps) {
			if ($a->{'name'} =~ /^\Q$v->{'prepend'}\E\-(.*)$/) {
				$name = $1."\@".$v->{'domain'};
				}
			}
		# XXX need to change alias types?
		push(@virts, { 'from' => $name,
			       'alias' => $a,
			       'to' => [ map { s/^\&//; $_ }
					       @{$a->{'values'}} ] });
		}
	return @virts;
	}
}

# delete_virtuser(&virtuser)
# Deletes a virtual mail user mapping
sub delete_virtuser
{
&require_mail();
if ($config{'mail_system'} == 1) {
	# Delete from sendmail
	if ($_[0]->{'alias'}) {
		# Delete alias too
		&sendmail::delete_alias($_[0]->{'alias'});
		}
	&sendmail::delete_virtuser($_[0]->{'virt'}, $sendmail_vfile,
				   $sendmail_vdbm, $sendmail_vdbmtype);
	}
elsif ($config{'mail_system'} == 0) {
	# Delete from postfix file
	if ($_[0]->{'alias'}) {
		# Delete alias too
		&postfix::delete_alias($_[0]->{'alias'});
		&postfix::regenerate_aliases();
		}
	&postfix::delete_mapping($virtual_type, $_[0]->{'virt'});
	&postfix::regenerate_virtual_table();
	}
elsif ($config{'mail_system'} == 2) {
	# Just delete the qmail alias
	&qmailadmin::delete_alias($_[0]->{'alias'});
	}
}

# modify_virtuser(&old, &new)
sub modify_virtuser
{
&require_mail();
local @to = @{$_[1]->{'to'}};
if ($config{'mail_system'} == 1) {
	# Modify in sendmail
	local $alias = $_[0]->{'alias'};
	if (&needs_alias(@to) && !$alias) {
		# Alias needs to be created and virtuser updated
		$_[1]->{'from'} =~ /^(\S*)\@(\S+)$/;
		local $an = ($1 || "default")."-".$2;
		local $alias = { "name" => $an,
				 "enabled" => 1,
				 "values" => \@to };
		&sendmail::create_alias($alias, $sendmail_afiles);
		local $virt = { "from" => $_[1]->{'from'},
				"to" => $an };
		&sendmail::modify_virtuser($_[0]->{'virt'}, $virt,
					   $sendmail_vfile, $sendmail_vdbm,
					   $sendmail_vdbmtype);
		$_[0]->{'virt'} = $virt;
		}
	elsif ($alias) {
		# Just update alias and maybe virtuser
		$alias->{'values'} = \@to;
		&sendmail::modify_alias($alias, $alias);
		if ($_[1]->{'from'} ne $_[0]->{'from'}) {
			# Re-named .. need to change too
			local $virt = { "from" => $_[1]->{'from'},
					"to" => $_[0]->{'virt'}->{'to'} };
			&sendmail::modify_virtuser($_[0]->{'virt'}, $virt,
						   $sendmail_vfile,
						   $sendmail_vdbm,
						   $sendmail_vdbmtype);
			$_[0]->{'virt'} = $virt;
			}
		}
	else {
		# Just update virtuser
		local $virt = { "from" => $_[1]->{'from'},
				"to" => $_[1]->{'to'}->[0] };
		&sendmail::modify_virtuser($_[0]->{'virt'}, $virt,
					   $sendmail_vfile, $sendmail_vdbm,
					   $sendmail_vdbmtype);
		$_[0]->{'virt'} = $virt;
		}
	}
elsif ($config{'mail_system'} == 0) {
	# Modify in postfix file
	local $alias = $_[0]->{'alias'};
	if (&needs_alias(@to) && !$alias) {
		# Alias needs to be created and virtuser updated
		$_[0]->{'from'} =~ /^(\S*)\@(\S+)$/;
		local $an = ($1 || "default")."-".$2;
		local $alias = { "name" => $an,
				 "enabled" => 1,
				 "values" => \@to };
		&postfix::create_alias($alias, $postfix_afiles);
		&postfix::regenerate_aliases();
		local $virt = { "name" => $_[1]->{'from'},
				"value" => $an };
		&postfix::modify_mapping($virtual_type, $_[0]->{'virt'}, $virt);
		$_[0]->{'virt'} = $virt;
		&postfix::regenerate_virtual_table();
		}
	elsif ($alias) {
		# Just update alias
		$alias->{'values'} = \@to;
		&postfix::modify_alias($alias, $alias);
		&postfix::regenerate_aliases();
		if ($_[1]->{'from'} ne $_[0]->{'from'}) {
			# Re-named .. need to change virtuser too
			local $virt = { "name" => $_[1]->{'from'},
					"value" => $_[1]->{'virt'}->{'value'} };
			&postfix::modify_mapping($virtual_type, $_[0]->{'virt'},
						 $virt);
			$_[0]->{'virt'} = $virt;
			&postfix::regenerate_virtual_table();
			}
		}
	else {
		# Just update virtuser
		local $virt = { "name" => $_[1]->{'from'},
				"value" => $_[1]->{'to'}->[0] };
		&postfix::modify_mapping($virtual_type, $_[0]->{'virt'}, $virt);
		$_[0]->{'virt'} = $virt;
		&postfix::regenerate_virtual_table();
		}
	}
elsif ($config{'mail_system'} == 2) {
	# Just update the qmail alias
	$_[1]->{'from'} =~ /^(\S*)\@(\S+)$/;
	local ($box, $dom) = ($1, $2);
	local ($virtmap) = grep { $_->{'domain'} eq $dom && !$_->{'user'} }
			        &qmailadmin::list_virts();
	local $alias = { 'name' => "$virtmap->{'prepend'}-$box",
			 'values' => \@to };
	&qmailadmin::modify_alias($_[0]->{'alias'}, $alias);
	$_[1]->{'alias'} = $alias;
	}
}

# create_virtuser(&virtuser)
# Creates a new virtual mail mapping
sub create_virtuser
{
&require_mail();
local @to = @{$_[0]->{'to'}};
if ($config{'mail_system'} == 1) {
	# Create in sendmail
	local $virt;
	if (&needs_alias(@to)) {
		# Need to create an alias, named address-domain
		$_[0]->{'from'} =~ /^(\S*)\@(\S+)$/;
		local $an = ($1 || "default")."-".$2;
		local $alias = { "name" => $an,
				 "enabled" => 1,
				 "values" => \@to };
		&sendmail::create_alias($alias, $sendmail_afiles);
		$virt = { "from" => $_[0]->{'from'},
			  "to" => $an };
		}
	else {
		# A single virtuser will do
		$virt = { "from" => $_[0]->{'from'},
			  "to" => $_[0]->{'to'}->[0] };
		}
	&sendmail::create_virtuser($virt, $sendmail_vfile,
				   $sendmail_vdbm,
				   $sendmail_vdbmtype);
	$_[0]->{'virt'} = $virt;
	}
elsif ($config{'mail_system'} == 0) {
	# Create in postfix file
	if (&needs_alias(@to)) {
		# Need to create an alias, named address-domain
		$_[0]->{'from'} =~ /^(\S*)\@(\S+)$/;
		local $an = ($1 || "default")."-".$2;
		local $alias = { "name" => $an,
				 "enabled" => 1,
				 "values" => \@to };
		&postfix::create_alias($alias, $postfix_afiles);
		&postfix::regenerate_aliases();
		$virt = { 'name' => $_[0]->{'from'},
			  'value' => $an };
		}
	else {
		# A single virtuser will do
		$virt = { 'name' => $_[0]->{'from'},
			  'value' => $_[0]->{'to'}->[0] };
		}
	&postfix::create_mapping($virtual_type, $virt);
	&postfix::regenerate_virtual_table();
	$_[0]->{'virt'} = $virt;
	}
elsif ($config{'mail_system'} == 2) {
	# Create a single Qmail alias
	$_[0]->{'from'} =~ /^(\S*)\@(\S+)$/;
	local ($box, $dom) = ($1, $2);
	local ($virtmap) = grep { $_->{'domain'} eq $dom && !$_->{'user'} }
			        &qmailadmin::list_virts();
	local $alias = { 'name' => "$virtmap->{'prepend'}-$box",
			 'values' => \@to };
	&qmailadmin::create_alias($alias);
	$_[0]->{'alias'} = $alias;
	}
}

# needs_alias(list..)
sub needs_alias
{
return 1 if (@_ != 1);
local $t;
foreach $t (@_) {
	return 1 if (&alias_type($t) != 1);
	}
return 0;
}

# join_alias(list..)
sub join_alias
{
return join(',', map { /\s/ ? "\"$_\"" : $_ } @_);
}

# is_mail_running()
# Returns 1 if the configured mail server is running, 0 if not
sub is_mail_running
{
&require_mail();
if ($config{'mail_system'} == 1) {
	# Check the sendmail PID
	local $pid;
	open(PID, $sconfig{'sendmail_pid'}) || return 0;
	chop($pid = <PID>);
	close(PID);
	return kill(0, $pid);
	}
elsif ($config{'mail_system'} == 0) {
	# Call the postfix module 
	return &postfix::is_postfix_running();
	}
elsif ($config{'mail_system'} == 2) {
	# Just look for qmail-send
	local ($pid) = &find_byname("qmail-send");
	return $pid ? 1 : 0;
	}
}

# shutdown_mail_server()
# Shuts down the mail server, or calls &error
sub shutdown_mail_server
{
&require_mail();
if ($config{'mail_system'} == 1) {
	# Kill or stop sendmail
	if ($sconfig{'sendmail_stop_command'}) {
		$out = &backquote_logged("$sconfig{'sendmail_stop_command'} </dev/null 2>&1");
		if ($?) { &error("<pre>$out</pre>"); }
		}
	else {
		local $pid;
		open(PID, $sconfig{'sendmail_pid'}) ||
			&error($text{'mstop_edown'});
		chop($pid = <PID>);
		close(PID);
		&kill_logged('KILL', $pid) || &error($text{'mstop_edown'});
		unlink($sconfig{'sendmail_pid'});
		}
	}
else {
	# Run the postfix stop command
	$out = `$pconfig{'postfix_control_command'} -c $postfix::config_dir stop 2>&1`;
	if ($?) { &error("<tt>$out</tt>"); }
	}
}

# startup_mail_server()
# Starts up the mail server, or calls &error
sub startup_mail_server
{
&require_mail();
if ($config{'mail_system'} == 1) {
	# Run the sendmail start command
	$out = &backquote_logged("$sconfig{'sendmail_command'} </dev/null 2>&1");
	if ($?) { &error("<pre>$out</pre>"); }
	}
else {
	# Run the postfix start command
	$out = `$pconfig{'postfix_control_command'} -c $postfix::config_dir start 2>&1`;
	if ($?) { &error("<tt>$out</tt>"); }
	}
}

# delete_mail_file(&user)
sub delete_mail_file
{
&require_mail();
if ($config{'mail_system'} == 1) {
	# Just remove the Sendmail mail file
	unlink(&sendmail::user_mail_file($_[0]->{'user'}));
	}
elsif ($config{'mail_system'} == 0) {
	# Find out from Postfix which file to delete
	local $s = &postfix::postfix_mail_system();
	unlink(&postfix::postfix_mail_file($_[0]->{'user'}));
	}
elsif ($config{'mail_system'} == 2) {
	# Find out from Qmail which file to delete
	unlink(&qmailadmin::user_mail_dir($_[0]->{'user'}));
	}
}

# rename_mail_file(&user, oldname)
sub rename_mail_file
{
&require_mail();
if ($config{'mail_system'} == 1) {
	# Just rename the Sendmail mail file (if necessary)
	if ($sconfig{'mail_dir'}) {
		local $of = &sendmail::user_mail_file($_[1]);
		local $nf = &sendmail::user_mail_file($_[0]->{'user'});
		rename($of, $nf);
		}
	}
elsif ($config{'mail_system'} == 0) {
	# Find out from Postfix which file to rename (if necessary)
	local $s = &postfix::postfix_mail_system();
	if ($s == 0) {
		local $newumf = &postfix::postfix_mail_file($_[0]->{'user'});
		local $oldumf = &postfix::postfix_mail_file($_[1]);
		rename($oldumf, $newumf);
		}
	}
elsif ($config{'mail_system'} == 2) {
	# Just rename the Qmail mail file (if necessary)
	if ($qmconfig{'mail_system'} == 0 && $qmconfig{'mail_dir'}) {
		local $of = &qmailadmin::user_mail_file($_[1]);
		local $nf = &qmailadmin::user_mail_file($_[0]->{'user'});
		rename($of, $nf);
		}
	}
}

# mail_file_size(&user)
sub mail_file_size
{
&require_mail();
local $umf;
if ($config{'mail_system'} == 1) {
	# Just look at the Sendmail mail file
	$umf = &sendmail::user_mail_file($_[0]->{'user'});
	}
elsif ($config{'mail_system'} == 0) {
	# Find out from Postfix which file to check
	$umf = &postfix::postfix_mail_file($_[0]->{'user'});
	}
elsif ($config{'mail_system'} == 2) {
	# Find out from Qmail which file or dir to check
	$umf = &qmailadmin::user_mail_dir($_[0]->{'user'});
	}
if (-d $umf) {
	# Need to sum up a maildir-format directory
	local $sz = 0;
	foreach $d ("$umf/cur", "$umf/new", "$umf/tmp") {
		opendir(DIR, $d) || last;
		foreach $f (readdir(DIR)) {
			local @st = stat("$d/$f");
			$sz += $st[7];
			}
		closedir(DIR);
		}
	return ( $sz, $umf );
	}
else {
	# Just the size of a single mail file
	local @st = stat($umf);
	return ( int($st[7]), $umf );
	}
}

# mail_system_base()
# Returns the base directory under which user mail files can be found
sub mail_system_base
{
&require_mail();
if ($config{'mail_system'} == 1) {
	# Find out from sendmail module config
	if ($sconfig{'mail_dir'}) {
		return $sconfig{'mail_dir'};
		}
	}
elsif ($config{'mail_system'} == 0) {
	# Find out from postfix
	local @s = &postfix::postfix_mail_system();
	if ($s[0] == 0) {
		return $s[1];
		}
	}
elsif ($config{'mail_system'} == 2) {
	# Find out from qmail module config
	if ($qmconfig{'mail_system'} == 0 && $qmconfig{'mail_dir'}) {
		return $qmconfig{'mail_dir'};
		}
	}
# If we get here, assume that mail is under home dirs
local %uconfig = &foreign_config("useradmin");
return $home_base;
}

# read_mail_link(&user)
sub read_mail_link
{
local %acl;
&read_acl(\%acl);
if ($config{'mail_system'} == 1 && $acl{$base_remote_user,'sendmail'}) {
	return "/sendmail/list_mail.cgi?user=".$_[0]->{'user'};
	}
elsif ($config{'mail_system'} == 0 && $acl{$base_remote_user,'postfix'}) {
	return "/postfix/list_mail.cgi?user=".$_[0]->{'user'};
	}
elsif ($config{'mail_system'} == 2 && $acl{$base_remote_user,'qmailadmin'}) {
	return "/qmailadmin/list_mail.cgi?user=".$_[0]->{'user'};
	}
else {
	return undef;
	}
}

# postfix_installed()
# Returns 1 if postfix is installed
sub postfix_installed
{
return &foreign_installed("postfix", 1) == 2;
}

# sendmail_installed()
# Returns 1 if postfix is installed
sub sendmail_installed
{
return &foreign_installed("sendmail", 1) == 2;
}

# qmail_installed()
# Returns 1 if qmail is installed
sub qmail_installed
{
return &foreign_installed("qmailadmin", 1) == 2;
}



1;

