sub require_postgres
{
return if ($require_postgres++);
&foreign_require("postgresql", "postgresql-lib.pl");
%qconfig = &foreign_config("postgresql");
}

# check_postgres_clash(domain, db, user)
# Returns 1 if some PostgreSQL database already exists
sub check_postgres_clash
{
&require_postgres();
local @dblist = &postgresql::list_databases();
return &indexof($_[1], @dblist) >= 0;
}

# setup_postgres(&domain)
# Create a new PostgreSQL database and user
sub setup_postgres
{
&$first_print($text{'setup_postgres'});
&require_postgres();
&postgresql::execute_sql($qconfig{'basedb'}, "create database $_[0]->{'db'}");
local $pass = &postgresql::get_postgresql_version() >= 7 ? "'$_[0]->{'pass'}'"
							 : $_[0]->{'pass'};
&postgresql::execute_sql($qconfig{'basedb'}, "create user \"$_[0]->{'user'}\" with password $pass nocreatedb nocreateuser");
&$second_print($text{'setup_done'});
}

# modify_postgres(&domain, &olddomain)
# Change the PostgreSQL user's password if needed
sub modify_postgres
{
&require_postgres();
if ($_[0]->{'pass'} ne $_[1]->{'pass'}) {
	&$first_print($text{'save_postgrespass'});
	local $pass = &postgresql::get_postgresql_version() >= 7 ?
			"'$_[0]->{'pass'}'" : $_[0]->{'pass'};
	&postgresql::execute_sql($qconfig{'basedb'}, "alter user \"$_[0]->{'user'}\" with password $pass");
	&$second_print($text{'setup_done'});
	}
}

# delete_postgres(&domain)
# Delete the PostgreSQL database and user
sub delete_postgres
{
&$first_print($text{'delete_postgres'});
&require_postgres();
&postgresql::execute_sql($qconfig{'basedb'}, "drop database $_[0]->{'db'}");
&postgresql::execute_sql($qconfig{'basedb'}, "drop user \"$_[0]->{'user'}\"");
&$second_print($text{'setup_done'});
}

# disable_postgres(&domain)
# Invalidate the domain's PostgreSQL user
sub disable_postgres
{
&$first_print($text{'disable_postgres'});
&require_postgres();
local $date = localtime(0);
&postgresql::execute_sql($qconfig{'basedb'}, "alter user \"$_[0]->{'user'}\" valid until '$date'");
&$second_print($text{'setup_done'});
}

# enable_postgres(&domain)
# Validate the domain's PostgreSQL user
sub enable_postgres
{
&$first_print($text{'enable_postgres'});
&require_postgres();
&postgresql::execute_sql($qconfig{'basedb'}, "alter user \"$_[0]->{'user'}\" valid until 'Jan 1 2038'");
&$second_print($text{'setup_done'});
}

1;

