
$feature_depends{'webalizer'} = [ 'web' ];

sub require_webalizer
{
return if ($require_webalizer++);
&foreign_require("webalizer", "webalizer-lib.pl");
%wconfig = &foreign_config("webalizer");
}

# setup_webalizer(&domain)
# Setup the Webalizer module for this domain, and create a Cron job to run it
# XXX locking!
sub setup_webalizer
{
&$first_print($text{'setup_webalizer'});
&require_webalizer();

local $alog = &get_apache_log($_[0]->{'dom'});
if (!$alog) {
	&$second_print($text{'setup_nolog'});
	return;
	}

# Create directory for stats
local $hdir = $config{'html_dir'} || 'public_html';
local $stats = "$_[0]->{'home'}/$hdir/stats";
if (!-d $stats) {
	&system_logged("mkdir '$stats' 2>/dev/null");
	&system_logged("chmod 755 '$stats'");
	&system_logged("chown $_[0]->{'uid'}:$_[0]->{'ugid'} '$stats'");
	}

# Set up config for log in Webalizer module
if (!-r &webalizer::log_config_name($alog)) {
	$lconf = { 'dir' => $stats,
		   'sched' => 1,
		   'type' => 0,
		   'over' => 0,
		   'clear' => 1,
		   'user' => $_[0]->{'user'},
		   'mins' => int(rand()*60),
		   'hours' => 0,
		   'days' => '*',
		   'months' => '*',
		   'weekdays' => '*' };
	&webalizer::save_log_config($alog, $lconf);

	# Create a custom webalizer.conf for the site
	# Copy webalizer.conf into place for site, and update
	local $cfile = &webalizer::config_file_name($alog);
	system("cp $wconfig{'webalizer_conf'} $cfile");
	system("chown $_[0]->{'user'}:$_[0]->{'group'} $cfile");
	local $wconf = &webalizer::get_config($alog);
	&webalizer::save_directive($wconf, "HistoryName", "webalizer.hist");
	&webalizer::save_directive($wconf, "IncrementalName", "webalizer.current");
	&webalizer::save_directive($wconf, "Incremental", "yes");
	&webalizer::save_directive($wconf, "LogFile", $alog);
	&webalizer::save_directive($wconf, "HostName", $_[0]->{'dom'});
	&flush_file_lines();

	# Create a Cron job to process the log
	&foreign_require("cron", "cron-lib.pl");
	local $job = { 'user' => 'root',
		       'active' => 1,
		       'mins' => $lconf->{'mins'},
		       'hours' => $lconf->{'hours'},
		       'days' => $lconf->{'days'},
		       'months' => $lconf->{'months'},
		       'weekdays' => $lconf->{'weekdays'},
		       'command' => "$webalizer::cron_cmd $alog" };
	if (!-r $webalizer::cron_cmd) {
		&lock_file($webalizer::cron_cmd);
		&cron::create_wrapper($webalizer::cron_cmd,
				      "webalizer", "webalizer.pl");
		&unlock_file($webalizer::cron_cmd);
		}
	&cron::create_cron_job($job);
	}
&$second_print($text{'setup_done'});
}

# modify_webalizer(&domain, &olddomain)
# Nothing to do here
sub modify_webalizer
{
}

# delete_webalizer(&domain)
# Delete the Webalizer config files and Cron job
sub delete_webalizer
{
&$first_print($text{'delete_webalizer'});
&require_webalizer();
local $alog = &get_apache_log($_[0]->{'dom'});
return if (!$alog);

# Delete config files
#local $lfile = &webalizer::log_config_name($alog);
#unlink($lfile);
#local $cfile = &webalizer::config_file_name($alog);
#unlink($cfile);
 
# Turn off cron job for webalizer config
&foreign_require("cron", "cron-lib.pl");
foreach $j (&cron::list_cron_jobs()) {
        if ($j->{'command'} eq "$webalizer::cron_cmd $alog") {
                &cron::delete_cron_job($j);
                last;
                }
        }
&$second_print($text{'setup_done'});
}

# check_webalizer_clash()
# Does nothing, because the web clash check is all that is needed
sub check_webalizer_clash
{
return 0;
}

sub enable_webalizer
{
# Does nothing yet
}

sub disable_webalizer
{
# Does nothing yet
}

1;

